/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_3D_PRIMITIVES_H
#define LG_3D_PRIMITIVES_H

/*
 * All vertices are Vertex_rgba
 * Could/should we implemenet the same with other vertex formats ?
 *
 * Check with:
 * INFO_OUT("MAX(MAX(QUAD_LINES3D_N_VERTICES, CUBOID_LINES3D_N_VERTICES), XYZ_ARROWS_LINES3D_N_VERTICES) = %d\n",
 * 	MAX(MAX(QUAD_LINES3D_N_VERTICES, CUBOID_LINES3D_N_VERTICES), XYZ_ARROWS_LINES3D_N_VERTICES))
 * INFO_OUT("LINES3D_N_VERTICES_MAX = %d\n", LINES3D_N_VERTICES_MAX)
 */

#define QUAD_N_VERTICES			4	/* Num of vertices of a quad */
#define QUAD_LINES3D_N_VERTICES		8	/* Num of vertices for all lines to draw a quad */

#define CUBOID_N_VERTICES		8	/* Num of vertices of a cuboid */
#define CUBOID_LINES3D_N_VERTICES	24	/* Num of vertices for all lines to draw a cuboid */

#define XYZ_ARROWS_LINES3D_N_VERTICES	18	/* Num of vertices for all lines to draw an xyz_arrows */

#define LINES3D_N_VERTICES_MAX		24	/* === Must be big enough, adjust as necessary === */

#if QUAD_LINES3D_N_VERTICES > LINES3D_N_VERTICES_MAX
  #error Sth is wrong with STH_LINES3D_N_VERTICES stuff
#elif CUBOID_LINES3D_N_VERTICES > LINES3D_N_VERTICES_MAX
  #error Sth is wrong with STH_LINES3D_N_VERTICES stuff
#elif XYZ_ARROWS_LINES3D_N_VERTICES > LINES3D_N_VERTICES_MAX
  #error Sth is wrong with STH_LINES3D_N_VERTICES stuff
#endif

/*
 * Generic 3D line set vertex buffer (Vertex_rgba)
 *
 * Useful for generating VBO from shapes - can have:
 * - an array of Vertex_rgba with array max size = LINES3D_N_VERTICES_MAX
 * or:
 * - a pointer to a bigger array (dynamically generated) of Vertex_rgba
 *
 *	Draw with glDrawArrays(GL_LINE, 0, n_vertices)
 *
 * Max num of lines = LINES3D_N_VERTICES_MAX / 2
 *
 * The vertex buffer is either a *static* one (static array) or a *dynamic* one (malloc3() generated)
 * You can/should access it using sth like:
 *	buffer = dynamic_b ? vb_ptr : vb_array
 */
typedef struct {
	Vertex_rgba	vb_array[LINES3D_N_VERTICES_MAX];
	Vertex_rgba	*vb_ptr;
	uint32_t	n_vertices;
	zboolean	dynamic_b;
} Lines3D_VB;

typedef struct {
	Vertex_rgba	vertex[QUAD_N_VERTICES];
} LG_Quad;

typedef struct {
	Vertex_rgba	vertex[CUBOID_N_VERTICES];
} LG_Cuboid;

typedef struct {
	Vertex_rgba	vertex[XYZ_ARROWS_LINES3D_N_VERTICES];
} XYZ_Arrows;

LG_Quad			lg_quad(vec3_t, vec3_t, vec3_t, vec3_t, LG_Color_u);

LG_Quad			lg_centered_unit_quad(LG_Color_u);

LG_Quad			lg_quad_transform(LG_Quad, mat4_t);

LG_Cuboid		lg_cuboid(vec3_t, vec3_t, vec3_t, vec3_t, vec3_t, vec3_t, vec3_t, vec3_t, LG_Color_u);

zboolean		lg_shape_is_cuboid(LG_Cuboid *);

LG_Cuboid		lg_centered_unit_cube(LG_Color_u);

LG_Cuboid		lg_cuboid_transform(LG_Cuboid, mat4_t);

XYZ_Arrows		lg_xyz_arrows_LH(float, LG_Color_u, LG_Color_u, LG_Color_u);

XYZ_Arrows		lg_xyz_arrows_RH(float, LG_Color_u, LG_Color_u, LG_Color_u);

Lines3D_VB		lg_l3d_vb_from_quad(LG_Quad);

Lines3D_VB		lg_l3d_vb_from_cuboid(LG_Cuboid);

Lines3D_VB		lg_l3d_vb_from_arrows(XYZ_Arrows);

vec3_t			vec3_LH_to_RH(vec3_t);

vec3_t			vec3_RH_to_LH(vec3_t);

/* (all Vertex_rgba until here) */

/* MOSTLY DEPRECATED - should cleanup/reorganize/rewrite some of following stuff, which is a bit of a mess */

const Vertex_uv		*lg_cube_normalized_vertices_uv();

const unsigned short	*lg_cube_vertex_uv_indices();

size_t			lg_sizeof_cube_normalized_vertices_uv();

size_t			lg_sizeof_cube_vertex_uv_indices();

const Vertex_rgba	*lg_cube_normalized_vertices_rgba();

const unsigned short	*lg_wired_cube_vertex_rgba_indices();

size_t			lg_sizeof_cube_normalized_vertices_rgba();

size_t			lg_sizeof_wired_cube_vertex_rgba_indices();

#endif /* LG_3D_PRIMITIVES_H */
