/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_AUDIO_H
#define LG_AUDIO_H

/*
 * See: https://wiki.libsdl.org/SDL2_mixer/CategoryAPI
 *
 * Plus, from: https://github.com/libsdl-org/SDL_mixer/blob/SDL2/include/SDL_mixer.h
 *
 * The audio device frequency is specified in Hz; in modern times, 48000 is
 * often a reasonable default.
 *
 * The audio device format is one of SDL's AUDIO_* constants. AUDIO_S16SYS
 * (16-bit audio) is probably a safe default. More modern systems may prefer
 * AUDIO_F32SYS (32-bit floating point audio).
 *
 * The audio device channels are generally 1 for mono output, or 2 for stereo,
 * but the brave can try surround sound configs with 4 (quad), 6 (5.1), 7
 * (6.1) or 8 (7.1).
 *
 * The audio device's chunk size is the number of sample frames (one sample
 * per frame for mono output, two samples per frame in a stereo setup, etc)
 * that are fed to the device at once. The lower the number, the lower the
 * latency, but you risk dropouts if it gets too low. 2048 is often a
 * reasonable default, but your app might want to experiment with 1024 or
 * 4096.
 */

#define LG_AUDIO_FREQ		48000			/* High values reduce latency - previously 44100 */
#define LG_AUDIO_FORMAT		AUDIO_S16SYS		/* Previously AUDIO_F32SYS */
#define LG_AUDIO_N_CHANNELS	2
#define LG_AUDIO_BUF_SIZE	(2 * 1024)		/* Low values reduce latency - previously 1024 */

#define LG_AUDIO_MAX_VOL	MIX_MAX_VOLUME

#define SOUND_FILENAME_MAXLEN	(64 - 1)

typedef struct {
	char		file_name[SOUND_FILENAME_MAXLEN + 1];
	Mix_Chunk	*sample;			/* sample->volume in range [0, 128] */
	int		channel;			/* 8 available channels */
} LG_Sound;

typedef enum {
	LG_PLAY, LG_REPEAT, LG_WAIT, LG_STOP, LG_PAUSE, LG_RESUME
} lg_play_mode;

int		lg_init_audio_to_defaults();

int		lg_init_audio(int, uint16_t, int, int);

void		lg_free_audio();

int		lg_load_play_music(const char *, int);

zboolean	lg_music_is_playing();

void		lg_pause_music();

void		lg_resume_music();

void		lg_stop_music();

void		lg_free_music();

LG_Sound	*lg_sound_new(const char *);

void		lg_disable_cant_play_audio_file();

void		lg_enable_cant_play_audio_file();

int		lg_sound_play(lg_play_mode, LG_Sound *);

zboolean	lg_sound_is_playing(LG_Sound *);

void		lg_sound_channel_set_volume(LG_Sound *, float);

float		lg_sound_channel_get_volume(LG_Sound *);

void		lg_sound_free(LG_Sound *);

void		lg_audio_set_global_volume(float);

void		lg_audio_set_music_volume(float);

void		lg_audio_set_sounds_volume(float);

void		lg_audio_pause();

void		lg_audio_resume();

int		lg_audio_get_n_channels();

#endif /* LG_AUDIO_H */
