/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_ENV_H
#define LG_ENV_H

/**
 * === LibGame environnement struct ===
 *
 * This is also a spec that the code is supposed to implement
 * (should be like that for all comments).
 *
 * === LibGame 2D coords sys ===
 * - Usually = SDL coords sys (y axis goes down) != OpenGL coords sys (y axis goes up)
 * - SDL uses int's in [0 - win_w] x [0 - win_h] with origin at top left corner
 * - OpenGL uses float's in [-1.0f, 1.0f] x [-1.0f, 1.0f] with origin at center
 * - OpenGL win uses int's in [0 - gl_win w] x [0 - gl_win h] with origin at bottom left corner
 * (-> glViewport() and glScissor() coords sys is (0, 0) = bottom-left, in pixels)
 *
 * ============================================================================
 * To avoid endless confusion, always mention 2D coords sys in func definition
 * ============================================================================
 */
typedef struct {
	/*
	 * SDL display mode stuff
	 */
	int		sdl_display;
	SDL_DisplayMode	sdl_display_mode;
	int		sdl_convert_surf_format;		/* For all SDL_ConvertSurfaceFormat() ops */
	int		sdl_create_rgb_surf_bitdepth;		/* For all SDL_CreateRGBSurface() ops */
	int		r_size, g_size, b_size, a_size;
	uint32_t	r_mask, g_mask, b_mask, a_mask;		/* SDL interprets each pixel as a 32-bit integer */
	/*
	 * Screen dims
	 */
	int		screen_w;
	int		screen_h;
	/*
	 * OpenGL ES 2.0 stuff -> now 3.1
	 */
	SDL_Window	*gl_win;				/* Top SDL/GL win */
	SDL_GLContext	gl_context;
	/*
	 * Opengl ES version >= 2.0 -> now 3.1
	 */
	int		gles_maj_v;
	int		gles_min_v;
	/*
	 * Top win dims are virtual/logical dims in fullscreen mode.
	 * In fullscreen mode, we compute an optimal viewport_rect (with letterboxing)
	 * and use scaling, keeping the same aspect ratio (like SDL2 with render logical size set).
	 * In windowed mode, viewport_rect and gl_win rect have the same size.
	 */
	int		top_win_logical_w;	/* In fullscreen mode, logical w */
	int		top_win_logical_h;	/* In fullscreen mode, logical h */
	Rec2Di		viewport_rect;		/* THE DEFAULT VIEWPORT, COMPUTED DURING INIT, MUST *NOT* BE MODIFIED AFTERWARDS */
	zboolean	fullscreen;		/* Should always be set to TRUE on Android ? Seems it's expected from SDL */
	float		fullscreen_scale;	/* MIN(scale_x, scale_y), to keep aspect ratio */
	/*
	 * Texture max values
	 */
	int		max_combined_texture_image_units;
	int		max_texture_size;
	int		max_cubemap_texture_size;
	/*
	 * Background
	 */
	LG_Texture	*bg_uniq_tex;
	LG_Color_u	bg_color;		/* Always set after a call to lg_clear_bg() */
	int		bg_shift_x;		/* Not sure what the purpose was exactly */
	int		bg_shift_y;		/* Not sure what the purpose was exactly */
	/*
	 * App stuff
	 */
	char 		*app_name;
	char 		*app_cmd;
	char		*org_name_android;	/* Android-specific */
	char		*app_name_android;	/* Android-specific */
	char		*assets_dir;		/* App data dir on Linux, app assets on Android */
	char		*app_wr_dir;		/* App home dir on Linux, app-specific writable persistent dir on Android */
	char		*vbo_cache;
	char		*obj_cache;
	/*
	 * Mouse support
	 */
	zboolean	enable_mouse;
	/*
	 * OpenGL ES extensions support
	 */
	zboolean	oes_vao_extension;
	void		(*glGenVertexArraysOES)(GLsizei, GLuint *);
	void		(*glBindVertexArrayOES)(GLuint);
	void		(*glDeleteVertexArraysOES)(GLsizei, const GLuint *);
	void		(*glIsVertexArrayOES)(GLuint);
	zboolean	oes_element_index_uint_extension;
	zboolean	ext_draw_instanced_extension;
	zboolean	ext_texcompr_s3tc_extension;
	zboolean	oes_texcompr_etc1_rgb8_extension;
	zboolean	khr_texcompr_astc_ldr_extension;
	/*zboolean	khr_texcompr_astc_hdr_extension;
	zboolean	ext_texcompr_astc_decode_mode_extension;*/
	/*
	 * Other stuff
	 */
	LG_ErrorContext	last_error_context;
	float		luminosity_k;
	LG_Sound	**sounds;
} LG_Env;

#endif /* LG_ENV_H */
