/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_MESH_H
#define LG_MESH_H

#define LG_MESH_NAME_MAX_LEN	(64 - 1)
#define MAT_NAME_MAX_LEN	(64 - 1)
#define N_MAX_MAT_PER_OBJ	64		/* Or N_MAX_MAT_PER_MTL ? */
#define N_MAX_USEMTL_PER_OBJ	256		/* 4096 ? */

#define LG_FLOAT_EPSILON	(1e-6)		/* 1e-6 or 1e-8 ? */

#define	LG_VBO_CACHE		"vbo_cache"
#define MAX_FILE_EXT_LEN	64		/* Extension length must not exceed that */
#define LG_MESH_INFO_EXT	".info"
#define LG_MESH_VBO_EXT		".vbo"
#define LG_MESH_IBO_EXT		".ibo"

#define	LG_OBJ_CACHE		"obj_cache"	/* Only used on Android so far */

#define LG_OBJ_FILE_MAXSIZE	((size_t)50 * (size_t)(1024 * 1024))	/* 50 MiB */

#define CHECK_PATH(_z_)		INFO_OUT("DEBUG: [%s %d]: %s() -> %s\n", basename2(__FILE__), __LINE__, __func__, _z_)

#define LG_MESH_IS_RH		1
#define LG_MESH_IS_LH		2

/* Mesh original format, ie from OBJ/FBX file or whatever */
typedef enum {
	LG_MESH_OBJ,
	LG_MESH_FBX,
	LG_MESH_TERRAIN,
	LG_MESH_SKYBOX,
	LG_MESH_OTHER
} lg_mesh_type;

typedef struct {
	char		name[MAT_NAME_MAX_LEN + 1];
	uint32_t	indice;					/* Material face indice in VBO (uint32_t with GL_OES_element_index_uint extension) */
	LG_Texture	*texture;
	char		tex_path[LG_TEX_PATH_MAX_LEN + 1];	/* Used by lg_mesh_reload_mats() */
} LG_Material;

/* Axis-aligned bounding box */
typedef struct {
	float	min_x;
	float	max_x;
	float	min_y;
	float	max_y;
	float	min_z;
	float	max_z;
} LG_BBox;

/*
 * === Triangle mesh ===
 *
 * We want the same type sizes in binary files on all suppored platforms
 * (ie Linux and Android so far) to avoid portability issues, so we now
 * use fixed-size types (uint32_t, int32_t, ...) a lot for vertices structs
 * and buffer objects.
 *
 * In OBJ file:
 *	- 'mtllib'	-> obj_file.mtl
 *	- 'usemtl'	-> material name
 *
 * In MTL file:
 *	- 'newmtl'	-> material name
 *	- 'map_K*'	-> texture path
 */
typedef struct {
	int32_t		type;					/* lg_mesh_type */
	char		name[LG_MESH_NAME_MAX_LEN + 1];		/* OBJ/FBX basename */
	zboolean	skinned;
	/* Generated VBO and IBO */
	union {
	  Vertex_uv_n	*vbo_data;				/* VBO (with interleaved vertex data) - sizeof(Vertex_uv_n) = 24 */
	  Vertex_uvn_iw	*vbo_data_iw;				/* VBO (with interleaved vertex data) - sizeof(Vertex_uvn_iw) = 36 */
	  //Vertex_rgba_n *vbo_data_rgba_n;			// -> TO ASSIGN COLORS TO ELEVATIONS
	};
	uint32_t	*ibo_data;				/* IBO (indices start at 0) - sizeof(uint32_t) = 4 */
	uint32_t	vbo_size;				/* previously size_t */
	uint32_t	ibo_size;				/* previously size_t */
	/* Read from obj file */
	int32_t		n_v;					/* Num of coords vertices */
	int32_t		n_vt;					/* Num of texture coords */
	int32_t		n_vn;					/* Num of normals */
	int32_t		n_f;					/* Num of faces */
	/* Materials */
	char		mtl_file[LG_MESH_NAME_MAX_LEN + 1];	/* The MTL file path */
	int32_t		n_mat;					/* Num of materials (with textures) in MTL file */
	int32_t		n_usemtl;				/* Num of usemtl tags in OBJ file */
	LG_Material	materials[N_MAX_USEMTL_PER_OBJ];	/* Materials referenced by usemtl tags */
	LG_BBox		bbox;
	LG_Cuboid	b_cuboid;				/* Bounding cuboid */
	Lines3D_VB	b_cuboid_l3d_vb;			/* Lines3D_VB from bounding box cuboid */
	zboolean	xyz_normalized;
	double		normalize_k;
	zboolean	horiz_centered;				/* Horizontally centered -> x, z*/
	zboolean	vert_centered;				/* Vertically centered -> y */
	zboolean	vert_bottom;				/* Vertically at bottom -> y - set mesh origin at bottom, override do_vert_center */
} LG_Mesh;

/* Mesh loading flags */
typedef struct {
	zboolean	force_reload;
	zboolean	invert_z;
	zboolean	normalize_xyz;
	zboolean	horiz_center;
	zboolean	vert_center;
	zboolean	vert_bottom;
} LG_LoadMesh_Flags;

/* Helper #define's */
#define FORCE_RELOAD	TRUE
#define INVERT_Z	TRUE
#define NORMALIZE_XYZ	TRUE
#define HORIZ_CENTER	TRUE
#define VERT_CENTER	TRUE
#define VERT_BOTTOM	TRUE

LG_Mesh			*lg_load_vbo(const char *, zboolean, zboolean, zboolean, zboolean, zboolean, zboolean);

LG_Mesh			*lg_mesh_new_from_objfile(const char *, zboolean, zboolean, zboolean, zboolean, zboolean);

LG_Mesh			*lg_mesh_new_from_fbxfile(const char *, zboolean, zboolean, zboolean, zboolean, zboolean);

void			lg_mesh_free(LG_Mesh *);

void			lg_mesh_info(LG_Mesh *);

int			lg_obj_file_save_to_cache(void *, const char *, size_t, size_t *);

void			*lg_obj_file_open_from_cache(const char *);

int			lg_vbo_save_to_file(const char *, LG_Mesh *);

int			lg_vbo_load_from_file(const char *, LG_Mesh **);

int			lg_mesh_reload_mats(LG_Mesh *);

LG_LoadMesh_Flags	lg_loadmesh_flags(zboolean, zboolean, zboolean, zboolean, zboolean, zboolean);

#endif /* LG_MESH_H */
