/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_SCENE_GRAPH_H
#define LG_SCENE_GRAPH_H

#define SCENENODE_NAME_MAX_LEN		(64 - 1)
#define SCENENODE_CHILD_MAX_NUM		256

#define SCENE_GRAPH_MAX_DEPTH		32
#define SCENEGRAPH_PRINTOUT_MAX_DEPTH	8

typedef enum {
	ROOT,
	FIRST_OBJ,
	MESH,
	LINES3D,
	GRID,		/* Means lines3d_vb is dynamic */
	LANDSCAPE,	/* = Landscape root */
	SKYBOX,
	LIGHT,
	SHADER_ONLY,
	UNDEFINED
} lg_scenenode_type;

typedef struct LG_SceneNode LG_SceneNode;

struct LG_SceneNode {
	int			id;					/* User-defined - not unique */
	char			name[SCENENODE_NAME_MAX_LEN + 1];	/* User-defined - not unique */
	lg_scenenode_type	type;
	LG_SceneNode		*parent;
	LG_SceneNode		*child[SCENENODE_CHILD_MAX_NUM];
	unsigned int		n_child;
	LG_Mesh			*mesh;
	Lines3D_VB		lines3d_vb;				/* Line set vertex buffer and num of vertices */
	mat4_t			local_matrix0;				/* 'Private' member - how to 'hide' this stuff */
	mat4_t			world_matrix0;				/* 'Private' member - how to 'hide' this stuff */
	mat4_t			*local_matrix;				/* Adjust (TRS) the mesh/lines3d_vb in its local space */
	mat4_t			*world_matrix;				/* Put (TRS) the mesh/lines3d_vb in world space */
	LG_ShaderProg 		shader_prog;
	LG_ShaderULocs		shader_u_loc;				/* Shader uniforms locations */
	LG_VAO			vao;					/* Vertex Array Object */
	LG_Texture		*tex_1;					/* For textures other than thoses from mesh mtl file */
};

typedef struct {
	int			id;		/* Set to -1 on error */
	LG_Camera		*cam1;
	LG_Light		light1;
	Rec2Df			landscape_rec;
	LG_SceneNode		*root;
	LG_SceneNode		*grid;
	LG_SceneNode		*xyz_arrows;
} LG_Scene;

LG_SceneNode	*lg_scenenode_new(int, const char *, lg_scenenode_type);

void		lg_scenenode_add_child(LG_SceneNode *, LG_SceneNode *);

void		lg_scenenode_remove_child(LG_SceneNode *, LG_SceneNode *);

void		lg_scenenode_free(LG_SceneNode *);

/* Any good ? */
#define		lg_scenenode_free2(s_node) {lg_scenenode_free(s_node); s_node = NULL;}

LG_SceneNode	*lg_scenenode_get_root(LG_SceneNode *);

int		lg_scenenode_remove_and_free_leaf(LG_SceneNode *);

void		lg_scenenode_remove_and_free_all(LG_SceneNode *);

void		lg_traverse_scene_graph(LG_SceneNode *, int (*)(LG_SceneNode *));

void		lg_traverse_scene_graph_2(LG_SceneNode *, int (*)(LG_SceneNode *, LG_Scene *), LG_Scene *);

LG_SceneNode	*lg_scenenode_find_by_id(LG_SceneNode *, int);

LG_SceneNode	*lg_scenenode_find_by_name(LG_SceneNode *, const char *);

void		lg_scenenode_reset_local_matrix(LG_SceneNode *);

void		lg_scenenode_set_local_matrix(LG_SceneNode *, vec3_t, LG_Quat, vec3_t);

void		lg_scenenode_update_local_matrix(LG_SceneNode *, vec3_t, LG_Quat, vec3_t);

void		lg_scenenode_update_world_matrix_tree(LG_SceneNode *, mat4_t *);

void		lg_scenenode_set_world_matrix(LG_SceneNode *, vec3_t, LG_Quat, vec3_t);

void		lg_scenenode_attach_to_camera(LG_SceneNode *, const LG_Camera *, vec3_t, vec3_t);

int		lg_scenenode_clone_worldm_l3dvb(LG_SceneNode *, LG_SceneNode *);

void		lg_traverse_printout_scene_graph(LG_SceneNode *, zboolean);

void		lg_scenenode_info(LG_SceneNode *);

int		lg_scenenode_info2(LG_SceneNode *);

const char	*lg_scenenode_type_string(int);

#endif /* LG_SCENE_GRAPH_H */
