/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_UI_H
#define LG_UI_H

#ifndef ANDROID_V
#  define FLUSH_KEYB_OR_TOUCHSCREEN_BUF		{lg_flush_keyb_events();}	/* lg_flush_mouse_events() also ? */
#else
#  define FLUSH_KEYB_OR_TOUCHSCREEN_BUF		{lg_flush_touch_events();}
#endif

#ifndef ANDROID_V
#  define WAIT_FOR_PRESS_OR_TOUCH		{lg_wait_for_any_key_pressed();}
#else
#  define WAIT_FOR_PRESS_OR_TOUCH		{lg_wait_for_finger_down(NULL, NULL);}
#endif

#define SPECIAL_KEYS_REPEAT_DELAY		200	/* In ms */

/* LPK = Last Pressed_Key */
#define LPK_MOUSE_ACTIVITY			50000	/* Returned by lg_get_user_input() as last pressed key in case of mouse activity */

/*
 * Check this out:
 * Android touch screen
 *	typedef enum {
 *		LG_FINGER_DOWN_NO_LOC = 0,
 *		LG_FINGER_DOWN_LEFT = 1,
 *		LG_FINGER_DOWN_H_CENTER = 2,
 *		LG_FINGER_DOWN_RIGHT = 4,
 *		LG_FINGER_DOWN_BOTTOM = 8,
 *		LG_FINGER_DOWN_V_CENTER = 16,
 *		LG_FINGER_DOWN_TOP = 32
 *	} lg_finger_down_loc;
 *
 *\code
 *	      L     C     R
 *	   -------------------
 *	T  |     |     |     |
 *	   -------------------
 *	C  |     |     |     |
 *	   -------------------
 *	B  |     |     |     |
 *	   -------------------
 *
 *	TZ_LEFT
 *	TZ_H_CENTER
 *	TZ_RIGHT
 *	TZ_BOTTOM
 *	TZ_V_CENTER
 *	TZ_TOP
 *\endcode
 */

/*
 *	=== Aircraft/flight simulator specific stuff ===
 *
 *	yaw + pitch + roll = attitude (Yaw/Pitch/Roll = standard order)
 *	Other namings:
 *	yaw = heading, pan / pitch = elevation, tilt / roll = bank
 *
 *	From: https://www1.grc.nasa.gov/beginners-guide-to-aeronautics/airplane-parts-function/
 *
 *	Fight controls:
 *	=========================================================================
 *	yoke (left/right)	->	ailerons			-> roll
 *	yoke (forward/backward)	->	elevators (on horiz stabilizer)	-> pitch
 *	rudder pedals		->	rudder (on vert stabilizer)	-> yaw
 *	thrust lever(s)		->	thrust
 *	flaps			->	increase lift
 *	air brakes
 *	=========================================================================
 *
 *	trim (rudder/elevator)	->	reduces backpressure on the control surface
 *	slats and flaps		->	enables take-off and landing at slower speeds
 *	spoilers		->	breaking & helps with rolling when used with ailerons
 *
 *	yoke = manche
 *	rudder pedals = palonnier
 *	thrust levers = manettes de gas
 *	thrust = poussée
 *	drag = trainée
 *	lift = portance
 *	flap handle
 *
 *	attitude indicator = artificial horizon
 *
 *	straight-and-level flight is flight in which heading and altitude are constantly maintained
 */

typedef struct {
	/* Linux */
	uint8_t			arrow_keys_state;	/* LEFT, RIGHT, UP, DOWN */
	zboolean		kmod_ctrl;
	zboolean		kmod_shift;
	zboolean		kmod_alt;
	uint32_t		last_pressed_key;
	/* Android */
	lg_finger_down_loc	finger_down_loc;	/* Logical OR of values */
	float			x;
	float			y;
	/* Cam/object motions and rotations = -1, 0, 1 */
	int			move_forward;		/* Z transl */
	int			move_up;		/* Y transl - not an aircraft motion */
	int			cam_truck;		/* X transl - not an aircraft motion */
	/* Yaw/Pitch/Roll = -1, 0, 1 */			/* Aircraft/flight simulator standard order */
	int			yaw;
	int			pitch;
	int			roll;
	/* Orbit cam */
	int			h_orbit;
	int			v_orbit;
	/* Mouse inputs */
	zboolean		left_click;
	zboolean		right_click;
	int			mouse_x;
	int			mouse_y;
	int			dx;
	int			dy;
	int			dz_wheel;		/* Mouse wheel also set move_forward */
	float			thrust;
	/* Appply for keyboard and mouse */
	zboolean		slow_motion;
} LG_InputState;

/*
 * TZ/tz means Touch Zone (not Time Zone)
 */
typedef enum {
	TZ_LEFT,
	TZ_H_CENTER,
	TZ_RIGHT,
	TZ_BOTTOM,
	TZ_V_CENTER,
	TZ_TOP
} lg_touch_zone;

typedef struct {
	LG_Window	win;
	LG_Color_u	color;
	Rec2Di		area;
} LG_TouchZoneWindow;

#define TZW_ARROW_POINTS_NUM	7

typedef enum {
	TZW_RECTANGLE,
	TZW_RECT_CORNERS,
	TZW_ARROW_UP,
	TZW_ARROW_DOWN,
	TZW_ARROW_LEFT,
	TZW_ARROW_RIGHT,
	TZW_DOUBLE_ARROW_HORIZ,
	TZW_DOUBLE_ARROW_VERT,
} lg_tzw_type;

const char		**lg_camcontrols_keys_str();

uint32_t		lg_get_user_input(LG_InputState *, Rec2Di *[]);

LG_TouchZoneWindow	*lg_tzwin_new(const char *, LG_Color_u, int, int, TTF_Font *);

void			lg_tzwin_show(LG_TouchZoneWindow *, lg_tzw_type);

void			lg_tzwin_free(LG_TouchZoneWindow *);

Rec2Di			lg_tzwin_get_rect(LG_TouchZoneWindow *);

void			lg_draw_rect_corners(Rec2Di, LG_Color_u, int);

void			lg_draw_arrow(Rec2Di r, LG_Color_u c, lg_tzw_type);

void			lg_rectangle_to_arrow(Rec2Di, lg_tzw_type, Point2Di *[]);

void			lg_set_special_keys_repeat_delay(uint32_t);

void			lg_input_state_info(LG_InputState *);

#endif /* LG_UI_H */
