/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 *
 *	LibGame is a SDL2/OpenGL ES 3.1 2D/3D minimalist game engine, which doesn't
 *	attempt to do *everything* but does only certain things and does them *well*,
 *	while not preventing you from doing other things (huh ?)
 *
 *	This version supports both Linux Desktop and Android (with NDK).
 */

#ifndef LIBGAME_H
#define LIBGAME_H

#ifndef _GNU_SOURCE
  #define _GNU_SOURCE
#endif
#ifndef _ISOC11_SOURCE
  #define _ISOC11_SOURCE
#endif

#define LIBGAME_NAME			"LibGame"
#define LIBGAME_V_NUM			"0.4.0~beta-20250531"
#define LIBGAME_COPYRIGHT_STR		"Copyright (C) Emmanuel Thomas-Maurin 2012-2025 - All rights reserved"

#define LIBGAME_WEBSITE			"https://www.etmsoftware.com"	/*"https://www.aetm-games.com"*/
#define LIBGAME_DOWNLOAD_WEBSITE	LIBGAME_WEBSITE "/download.php"
#define LIBGAME_SUPPORT_WEBSITE		LIBGAME_WEBSITE "/help.php"
#define LIBGAME_SUPPORT_EMAIL		"support@etmsoftware.com"	/*"manu@manutm.net"*/

#if !defined(ANDROID_V) && !defined(WIN32_V)
  #define LINUX_V
#endif

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include <math.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <byteswap.h>
#include <stdalign.h>
#ifndef WIN32_V			/* Linux and Android versions */
  #include <grp.h>
  #include <pwd.h>
  #include <signal.h>
#else
  #include <direct.h>
#endif

#ifdef ANDROID_V
  #include <jni.h>
  #include <android/log.h>
#endif

#ifndef WIN32_V			/* Linux and Android versions */
  #include <SDL.h>
  #include <SDL_image.h>
  #include <SDL_mixer.h>
  #include <SDL_ttf.h>
  #include <SDL_syswm.h>
  #ifdef LINUX_V
    #include <SDL_mouse.h>
  #endif
#else				/* Not tested */
  #include <SDL.h>
  #include <SDL2/SDL_image.h>
  #include <SDL2/SDL_mixer.h>
  #include <SDL2/SDL_ttf.h>
  #include <SDL_mouse.h>
  #include <SDL_syswm.h>
#endif

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <GLES3/gl3.h>
#include <GLES3/gl3ext.h>

/* === For LibGame, use VERBOSE_OUTPUT and DEBUG_OUTPUT, *not* these token below === */

/* Logging stuff for libetm - must appear before #include libetm.h */
#define LIBETM_VERBOSE_OUTPUT
/*#define LIBETM_DEBUG_OUTPUT*/

/* LIBETM_DEBUG_OUTPUT implies LIBETM_VERBOSE_OUTPUT */
#ifdef LIBETM_DEBUG_OUTPUT
  #ifndef LIBETM_VERBOSE_OUTPUT
    #define LIBETM_VERBOSE_OUTPUT
  #endif
#endif

#ifdef ANDROID_V
  #include "../extra_libs/libetm-0.5.1/libetm.h"
#else
  #ifndef WIN32_V		/* Linux version */
    #include <libetm/libetm.h>
  #else
    #include "../extra_libs/libetm-0.5.1/libetm.h"
  #endif
#endif

#ifndef ANDROID_V
  #undef INFO_ERR
  #define INFO_ERR INFO_ERR2
#endif

/* === Now, the LibGame verbose and debug tokens are just below === */

#define VERBOSE_OUTPUT
//#define DEBUG_OUTPUT

#ifdef DEBUG_OUTPUT
  #ifndef VERBOSE_OUTPUT
    #define VERBOSE_OUTPUT
  #endif
#endif

#define TMPSTR_SIZE		((4 * 1024) - 1)	/* = 4 KiB - 1 */
#define FILE_NAME_MAXLEN	((2 * 1024) - 1)	/* = 2 KiB - 1 */

#ifdef LIBGAME_COMPILE_TIME
  #include "../extra_libs/math_3d.h"
  #include "../extra_libs/stb_image.h"
  #include "../extra_libs/ufbx.h"
#else
  #include <libgame/extra_libs/math_3d.h>
  #include <libgame/extra_libs/stb_image.h>
  #include <libgame/extra_libs/ufbx.h>
#endif

/* What order for header files ? */

#include "lg_vbo.h"
#include "lg_shader_progs.h"
#include "lg_vertex.h"
#include "lg_gr_func.h"
#include "lg_textures.h"
#include "lg_dds_loader.h"
#include "lg_astc_loader.h"
#include "lg_string.h"
#include "lg_sprites.h"
#include "lg_background.h"
#include "lg_wins.h"
#include "lg_keyboard.h"
#include "lg_mouse.h"
#include "lg_touchscreen.h"
#include "lg_audio.h"
#include "lg_dirs_stuff.h"
#include "lg_quaternions.h"
#include "lg_misc.h"
#include "lg_ui.h"
#include "lg_3d_primitives.h"
#include "lg_opengl_2d.h"
#include "lg_mesh.h"
#include "lg_camera.h"
#include "lg_obj_parser.h"
#include "lg_light.h"
#include "lg_scene_graph.h"
#include "lg_scene.h"
#include "lg_collision_detect.h"
#include "lg_admob.h"
#include "lg_vao_gl_ext.h"
#include "lg_cubemap.h"
#include "lg_error.h"
#include "lg_file_ops.h"
#include "lg_android_assets.h"
#include "lg_render.h"
#include "lg_terrain.h"
#include "lg_perlin_noise.h"
#include "lg_landscape.h"
#include "lg_fbx_parser.h"
#include "lg_cam_controls.h"
#include "lg_part_sys.h"
/* OK ? */
#include "lg_env.h"
#include "lg_env_instance.h"
#include "lg_math_extra.h"

typedef enum {
	LG_OK = LIBETM_LASTERRORCODE + 1,
	LG_ERROR,
	LG_INIT_ERROR,
	LG_EXTRA_LIB_INIT_ERROR,
	LG_OPENGL_ERROR,
	LG_GLSL_ERROR,
	LG_CREATE_FILE_ERROR,
	LG_OPEN_FILE_ERROR,
	LG_SAVE_FILE_ERROR,
	LG_READ_FROM_FILE_ERROR,
	LG_WRITE_TO_FILE_ERROR,
	LG_FILE_ACCESS_ERROR,
	LG_READ_FROM_FILE_EOF,
	LG_EOF,
	LG_OOM,
	LG_INVALID_PTR,
	/* Unix signals */
	LG_BUSERR,		/* SIGBUS	Bus error (bad memory access) */
	LG_FPE,			/* SIGFPE	Floating-point exception */
	LG_ILL,			/* SIGILL	Illegal Instruction */
	LG_SEGFAULT,		/* SIGSEGV	Invalid memory reference */
	/* Must really be the last one */
	LG_LASTERRORCODE
} lg_error_code;

typedef enum {
	LG_CONTINUE = LG_LASTERRORCODE + 1,
	LG_CANCEL,
	LG_QUIT,
	LG_WON,
	LG_LOST,
	LG_LASTRETURNCODE
} lg_return_code;

/*
 * WIN_LOGICAL_W and WIN_LOGICAL_H must be defined in app
 *	16/9 or (on new mobiles) 18/9 aspect ratio ?
 *	#define WIN_LOGICAL_W		1000
 *	#define WIN_LOGICAL_H		500
 * On Android, we *always* want/use landscape orientation
 */
#define LG_WIN_WIDTH_MIN		800
#define LG_WIN_HEIGHT_MIN		400

#define LG_WIN_D_HEIGHT_HACK		2	/* DIRTY HACK: We want the whole win to be visible */

/* Requested bit-depth */
#define LG_REQUESTED_RED_SIZE		5
#define LG_REQUESTED_GREEN_SIZE		6
#define LG_REQUESTED_BLUE_SIZE		5
#define LG_REQUESTED_ALPHA_SIZE		0
#define LG_REQUESTED_DEPTH_SIZE		16
#define LG_REQUESTED_STENCIL_SIZE	8

int		lg_init(int, int, const char *, const char *, const char *, const char *, const char *, const char *, const char *);

Rec2Di		lg_get_default_viewport();

void		lg_set_new_viewport(Rec2Di);

void		lg_reset_viewport();

void		lg_enable_mouse();

void		lg_disable_mouse();

void		lg_list_opengl_extensions();

zboolean	lg_load_engine_fonts();

void		lg_engine_fonts_info();

void		lg_quit(int);

void		lg_swap_fb();

void		lg_show_lib_info();

void		lg_show_sys_info();

void		lg_show_extra_sys_info(SDL_Window *);

zboolean	sdl2_is_installed();

/* Must be freed afterwards with l_str_free() */
char		*lg_get_sdl_win_flags(SDL_Window *);

#endif /* LIBGAME_H */
